@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Resource class containing all logic for creating, checking, and updating resources."""
 import logging
-from os import unlink
+from os import remove
 from os.path import join
 from typing import Optional, List, Tuple, Dict, Union
 
@@ -51,7 +51,8 @@ class Resource(HDXObject):
             'datastore_delete': 'datastore_delete',
             'datastore_create': 'datastore_create',
             'datastore_insert': 'datastore_insert',
-            'datastore_upsert': 'datastore_upsert'
+            'datastore_upsert': 'datastore_upsert',
+            'datastore_search': 'datastore_search'
         }
 
     def update_from_yaml(self, path=join('config', 'hdx_resource_static.yml')):
@@ -141,18 +142,14 @@ class Resource(HDXObject):
         """
         self.file_to_upload = file_to_upload
 
-    def check_required_fields(self, ignore_fields=list()):
-        # type: (List[str]) -> None
-        """Check that metadata for resource is complete and add resource_type and url_type if not supplied.
-        The parameter ignore_fields should be set if required to any fields that should be ignored for the particular
-        operation.
-
-        Args:
-            ignore_fields (List[str]): Fields to ignore. Default is [].
+    def check_url_filetoupload(self):
+        # type: () -> bool
+        """Check if url or file to upload provided for resource and add resource_type and url_type if not supplied.
 
         Returns:
-            None
+            bool: Whether url field and file_to_upload are both populated
         """
+        url_file_to_upload = False
         if self.file_to_upload is None:
             if 'url' in self.data:
                 if 'resource_type' not in self.data:
@@ -162,16 +159,45 @@ class Resource(HDXObject):
             else:
                 raise HDXError('Either a url or a file to upload must be supplied!')
         else:
-            if 'url' not in self.data:
-                self.data['url'] = 'ignore'  # must be set even though overwritten
+            if 'url' in self.data:
+                url_file_to_upload = True
             if 'resource_type' not in self.data:
                 self.data['resource_type'] = 'file.upload'
             if 'url_type' not in self.data:
                 self.data['url_type'] = 'upload'
             if 'tracking_summary' in self.data:
                 del self.data['tracking_summary']
+        return url_file_to_upload
+
+    def check_required_fields(self, ignore_fields=list()):
+        # type: (List[str]) -> None
+        """Check that metadata for resource is complete. The parameter ignore_fields should be set if required to
+        any fields that should be ignored for the particular operation.
+
+        Args:
+            ignore_fields (List[str]): Fields to ignore. Default is [].
+
+        Returns:
+            None
+        """
+        self.check_url_filetoupload()
         self._check_required_fields('resource', ignore_fields)
 
+    def _remove_url(self, url_file_to_upload):
+        # type: (bool) -> None
+        """If input parameter is True, remove url from data.
+
+        Args:
+            url_file_to_upload (bool): Whether to remove url
+
+        Returns:
+            None
+        """
+        if url_file_to_upload:
+            logger.warning(
+                'Either a url or a file to upload must be supplied not both! Deleting url from resource %s!' % self.data['id'])
+            del self.data['url']
+
     def update_in_hdx(self):
         # type: () -> None
         """Check if resource exists in HDX and if so, update it
@@ -179,6 +205,7 @@ class Resource(HDXObject):
         Returns:
             None
         """
+        self._remove_url(self.check_url_filetoupload())
         self._update_in_hdx('resource', 'id', self.file_to_upload)
 
     def create_in_hdx(self):
@@ -188,7 +215,17 @@ class Resource(HDXObject):
         Returns:
             None
         """
-        self._create_in_hdx('resource', 'id', 'name', self.file_to_upload)
+        url_file_to_upload = self.check_url_filetoupload()
+        id = self.data.get('id')
+        if id and self._load_from_hdx('resource', id):
+            logger.warning('%s exists. Updating %s' % ('resource', id))
+            self._remove_url(url_file_to_upload)
+            self._merge_hdx_update('resource', 'id', self.file_to_upload)
+        else:
+            if url_file_to_upload:
+                raise HDXError('Either a url or a file to upload must be supplied not both!')
+            self._check_required_fields('resource', list())
+            self._save_to_hdx('create', 'name', self.file_to_upload)
 
     def delete_from_hdx(self):
         # type: () -> None
@@ -264,6 +301,44 @@ class Resource(HDXObject):
             path = downloader.download_file(url, folder, filename)
             return url, path
 
+    @staticmethod
+    def get_all_resource_ids_in_datastore(configuration=None):
+        # type: (Optional[Configuration]) -> List[str]
+        """Get list of resources that have a datastore returning their ids.
+
+        Args:
+            configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
+
+        Returns:
+            List[str]: List of resource ids that are in the datastore
+        """
+        resource = Resource(configuration=configuration)
+        success, result = resource._read_from_hdx('datastore', '_table_metadata', 'resource_id',
+                                                  Resource.actions()['datastore_search'], limit=10000)
+        resource_ids = list()
+        if not success:
+            logger.debug(result)
+        else:
+            for record in result['records']:
+                resource_ids.append(record['name'])
+        return resource_ids
+
+    def has_datastore(self):
+        # type: () -> bool
+        """Check if the resource has a datastore.
+
+        Returns:
+            bool: Whether the resource has a datastore or not
+        """
+        success, result = self._read_from_hdx('datastore', self.data['id'], 'resource_id',
+                                              self.actions()['datastore_search'])
+        if not success:
+            logger.debug(result)
+        else:
+            if result:
+                return True
+        return False
+
     def delete_datastore(self):
         # type: () -> None
         """Delete a resource from the HDX datastore
@@ -280,7 +355,7 @@ class Resource(HDXObject):
     def create_datastore(self, schema=None, primary_key=None,
                          delete_first=0, path=None):
         # type: (Optional[List[Dict]], Optional[str], int, Optional[str]) -> None
-        """For csvs, create a resource in the HDX datastore which enables data preview in HDX. If no schema is provided
+        """For tabular data, create a resource in the HDX datastore which enables data preview in HDX. If no schema is provided
         all fields are assumed to be text. If path is not supplied, the file is first downloaded from HDX.
 
         Args:
@@ -350,11 +425,11 @@ class Resource(HDXObject):
                 raisefrom(HDXError, 'Upload to datastore of %s failed!' % url, e)
             finally:
                 if delete_after_download:
-                    unlink(path)
+                    remove(path)
 
     def create_datastore_from_dict_schema(self, data, delete_first=0, path=None):
         # type: (dict, int, Optional[str]) -> None
-        """For csvs, create a resource in the HDX datastore which enables data preview in HDX from a dictionary
+        """For tabular data, create a resource in the HDX datastore which enables data preview in HDX from a dictionary
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -373,7 +448,7 @@ class Resource(HDXObject):
     def create_datastore_from_yaml_schema(self, yaml_path, delete_first=0,
                                           path=None):
         # type: (str, Optional[int], Optional[str]) -> None
-        """For csvs, create a resource in the HDX datastore which enables data preview in HDX from a YAML file
+        """For tabular data, create a resource in the HDX datastore which enables data preview in HDX from a YAML file
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -390,7 +465,7 @@ class Resource(HDXObject):
 
     def create_datastore_from_json_schema(self, json_path, delete_first=0, path=None):
         # type: (str, int, Optional[str]) -> None
-        """For csvs, create a resource in the HDX datastore which enables data preview in HDX from a JSON file
+        """For tabular data, create a resource in the HDX datastore which enables data preview in HDX from a JSON file
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -407,7 +482,7 @@ class Resource(HDXObject):
 
     def create_datastore_for_topline(self, delete_first=0, path=None):
         # type: (int, Optional[str]) -> None
-        """For csvs, create a resource in the HDX datastore which enables data preview in HDX using the built in
+        """For tabular data, create a resource in the HDX datastore which enables data preview in HDX using the built in
         YAML definition for a topline. If path is not supplied, the file is first downloaded from HDX.
 
         Args:
@@ -423,7 +498,7 @@ class Resource(HDXObject):
     def update_datastore(self, schema=None, primary_key=None,
                          path=None):
         # type: (Optional[List[Dict]], Optional[str], Optional[str]) -> None
-        """For csvs, update a resource in the HDX datastore which enables data preview in HDX. If no schema is provided
+        """For tabular data, update a resource in the HDX datastore which enables data preview in HDX. If no schema is provided
         all fields are assumed to be text. If path is not supplied, the file is first downloaded from HDX.
 
         Args:
@@ -438,7 +513,7 @@ class Resource(HDXObject):
 
     def update_datastore_from_dict_schema(self, data, path=None):
         # type: (dict, Optional[str]) -> None
-        """For csvs, update a resource in the HDX datastore which enables data preview in HDX from a dictionary
+        """For tabular data, update a resource in the HDX datastore which enables data preview in HDX from a dictionary
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -453,7 +528,7 @@ class Resource(HDXObject):
 
     def update_datastore_from_yaml_schema(self, yaml_path, path=None):
         # type: (str, Optional[str]) -> None
-        """For csvs, update a resource in the HDX datastore which enables data preview in HDX from a YAML file
+        """For tabular data, update a resource in the HDX datastore which enables data preview in HDX from a YAML file
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -468,7 +543,7 @@ class Resource(HDXObject):
 
     def update_datastore_from_json_schema(self, json_path, path=None):
         # type: (str, Optional[str]) -> None
-        """For csvs, update a resource in the HDX datastore which enables data preview in HDX from a JSON file
+        """For tabular data, update a resource in the HDX datastore which enables data preview in HDX from a JSON file
         containing a list of fields and types of form {'id': 'FIELD', 'type': 'TYPE'} and optionally a primary key.
         If path is not supplied, the file is first downloaded from HDX.
 
@@ -483,7 +558,7 @@ class Resource(HDXObject):
 
     def update_datastore_for_topline(self, path=None):
         # type: (Optional[str]) -> None
-        """For csvs, update a resource in the HDX datastore which enables data preview in HDX using the built in YAML
+        """For tabular data, update a resource in the HDX datastore which enables data preview in HDX using the built in YAML
         definition for a topline. If path is not supplied, the file is first downloaded from HDX.
 
         Args:
@@ -501,7 +576,7 @@ class Resource(HDXObject):
         Returns:
             None
         """
-        self._read_from_hdx('resource', self.data['id'], action=self.actions()['patch'])
+        self._write_to_hdx('patch', self.data, 'id')
 
     def get_resource_views(self):
         # type: () -> List[ResourceView]
@@ -608,3 +683,22 @@ class Resource(HDXObject):
                 if not found:
                     raise HDXError('No resource views have title %s in this resource!' % title)
         resource_view.delete_from_hdx()
+
+    def enable_dataset_preview(self):
+        # type: () -> None
+        """Enable dataset preview of resource
+
+        Returns:
+            None
+        """
+        self.data['dataset_preview_enabled'] = 'True'
+
+    def disable_dataset_preview(self):
+        # type: () -> None
+        """Disable dataset preview of resource
+
+        Returns:
+            None
+        """
+        self.data['dataset_preview_enabled'] = 'False'
+
