@@ -4,8 +4,8 @@
 import copy
 import datetime
 import json
-import os
 import tempfile
+from os import remove
 from os.path import join
 
 import pytest
@@ -19,10 +19,12 @@ from hdx.data.organization import Organization
 from hdx.data.resource import Resource
 from hdx.data.user import User
 from hdx.hdx_configuration import Configuration
+from hdx.hdx_tagscleanup import Tags, ChainRuleError
 from . import MockResponse, user_data, organization_data
 from .test_organization import organization_mockshow
 from .test_showcase import showcase_resultdict
 from .test_user import user_mockshow
+from .test_resource_view import resource_view_list
 
 resulttags = [{'state': 'active', 'display_name': 'conflict', 'vocabulary_id': None,
                'id': '1dae41e5-eacd-4fa5-91df-8d80cf579e53', 'name': 'conflict'},
@@ -295,6 +297,10 @@ class TestDataset:
                     return mockshow(url, datadict)
                 if 'hxl' in url:
                     return mockhxlupdate(url, datadict)
+                if 'default' in url:
+                    result = json.dumps(resource_view_list)
+                    return MockResponse(200,
+                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_create_default_resource_views"}' % result)
                 if 'resource' in url:
                     result = json.dumps(TestDataset.resources_data[0])
                     return MockResponse(200,
@@ -331,6 +337,10 @@ class TestDataset:
                     return mockshow(url, datadict)
                 if 'hxl' in url:
                     return mockhxlupdate(url, datadict)
+                if 'default' in url:
+                    result = json.dumps(resource_view_list)
+                    return MockResponse(200,
+                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_create_default_resource_views"}' % result)
                 if 'resource' in url:
                     result = json.dumps(TestDataset.resources_data[0])
                     return MockResponse(200,
@@ -370,6 +380,29 @@ class TestDataset:
 
         Configuration.read().remoteckan().session = MockSession()
 
+    @pytest.fixture(scope='function')
+    def post_reorder(self):
+        class MockSession(object):
+            @staticmethod
+            def post(url, data, headers, files, allow_redirects, auth):
+                decodedata = data.decode('utf-8')
+                datadict = json.loads(decodedata)
+                if 'show' in url:
+                    return mockshow(url, datadict)
+                if 'hxl' in url:
+                    return mockhxlupdate(url, datadict)
+                if 'reorder' not in url:
+                    return MockResponse(404,
+                                        '{"success": false, "error": {"message": "TEST ERROR: Not reorder", "__type": "TEST ERROR: Not Reorder Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_resource_reorder"}')
+                if datadict['id'] == '6f36a41c-f126-4b18-aaaf-6c2ddfbc5d4d':
+                    return MockResponse(200,
+                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_resource_reorder"}' % decodedata)
+
+                return MockResponse(404,
+                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=package_resource_reorder"}')
+
+        Configuration.read().remoteckan().session = MockSession()
+
     @pytest.fixture(scope='function')
     def post_delete(self):
         class MockSession(object):
@@ -379,7 +412,7 @@ class TestDataset:
                 datadict = json.loads(decodedata)
                 if 'show' in url:
                     return mockshow(url, datadict)
-                if 'delete' not in url:
+                if 'purge' not in url and 'delete' not in url:
                     return MockResponse(404,
                                         '{"success": false, "error": {"message": "TEST ERROR: Not delete", "__type": "TEST ERROR: Not Delete Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=dataset_delete"}')
                 if 'resource' in url:
@@ -514,7 +547,7 @@ class TestDataset:
             dataset.create_in_hdx()
 
         config = Configuration(hdx_read_only=True)
-        config.setup_remoteckan()
+        config.setup_remoteckan(user_agent='test')
         config.remoteckan().session = Configuration.read().remoteckan().session
         uniqueval = 'myconfig'
         config.unique = uniqueval
@@ -542,7 +575,7 @@ class TestDataset:
         resource.set_file_to_upload(file.name)
         dataset.add_update_resource(resource)
         dataset.create_in_hdx()
-        os.unlink(file.name)
+        remove(file.name)
         assert len(dataset.resources) == 2
         # Dataset creates that end up updating are in the test below
 
@@ -644,7 +677,25 @@ class TestDataset:
         resource.set_file_to_upload(file.name)
         dataset.update_in_hdx(update_resources_by_name=False)
         assert len(dataset.resources) == 3
-        os.unlink(file.name)
+        remove(file.name)
+        dataset = Dataset(dataset_data)
+        resources_data = copy.deepcopy(TestDataset.resources_data)
+        resource = Resource(resources_data[0])
+        dataset.add_update_resource(resource)
+        dataset.update_in_hdx(remove_additional_resources=False)
+        assert len(dataset.resources) == 2
+        dataset = Dataset(dataset_data)
+        resources_data = copy.deepcopy(TestDataset.resources_data)
+        resource = Resource(resources_data[0])
+        dataset.add_update_resource(resource)
+        dataset.update_in_hdx(remove_additional_resources=True)
+        assert len(dataset.resources) == 1
+        dataset = Dataset(dataset_data)
+        resources_data = copy.deepcopy(TestDataset.resources_data)
+        resource = Resource(resources_data[0])
+        dataset.add_update_resource(resource)
+        dataset.update_in_hdx(update_resources_by_name=False, remove_additional_resources=True)
+        assert len(dataset.resources) == 1
 
     def test_delete_from_hdx(self, configuration, post_delete):
         dataset = Dataset.read_from_hdx('TEST1')
@@ -739,6 +790,13 @@ class TestDataset:
         with pytest.raises(HDXError):
             dataset.delete_resource('NOTEXIST')
 
+    def test_reorder_resources(self, configuration, post_reorder):
+        dataset = Dataset.read_from_hdx('TEST1')
+        dataset.reorder_resources(['3d777226-96aa-4239-860a-703389d16d1f', 'de6549d8-268b-4dfe-adaf-a4ae5c8510d5'])
+        del dataset['id']
+        with pytest.raises(HDXError):
+            dataset.reorder_resources(['3d777226-96aa-4239-860a-703389d16d1f', 'de6549d8-268b-4dfe-adaf-a4ae5c8510d5'])
+
     def test_search_in_hdx(self, configuration, search):
         datasets = Dataset.search_in_hdx('ACLED')
         assert len(datasets) == 10
@@ -763,6 +821,8 @@ class TestDataset:
             Dataset.get_all_datasets(limit=11)
         with pytest.raises(HDXError):
             Dataset.get_all_datasets(page_size=5)  # test repeated dataset (see mockall)
+        datasets = Dataset.get_all_datasets(page_size=5, check_duplicates=False)  # test repeated dataset (see mockall)
+        assert len(datasets) == 6
 
     def test_get_all_resources(self, configuration, search):
         datasets = Dataset.search_in_hdx('ACLED')
@@ -889,6 +949,25 @@ class TestDataset:
         dataset.add_tag('LALA')
         assert dataset['tags'] == [{'name': 'LALA'}]
         assert dataset.get_tags() == ['LALA']
+        dataset.add_tag(u'LALA')
+        assert dataset.get_tags() == ['LALA']
+        dataset.add_tag(u'HAHA')
+        assert dataset.get_tags() == ['LALA', u'HAHA']
+        dataset.remove_tag(u'LALA')
+        assert dataset.get_tags() == [u'HAHA']
+        dataset.add_tag('HAHA')
+        assert dataset.get_tags() == [u'HAHA']
+
+    def test_is_set_subnational(self, read):
+        dataset = Dataset.read_from_hdx('TEST1')
+        assert dataset['subnational'] == '1'
+        assert dataset.is_subnational() is True
+        dataset.set_subnational(False)
+        assert dataset['subnational'] == '0'
+        assert dataset.is_subnational() is False
+        dataset.set_subnational(True)
+        assert dataset['subnational'] == '1'
+        assert dataset.is_subnational() is True
 
     def test_get_add_location(self, locations, read):
         Country.countriesdata(use_live=False)
@@ -1053,3 +1132,83 @@ class TestDataset:
             dataset.remove_filetype('csv')
         assert dataset.get_filetypes() == ['xlsx', 'csv']
 
+    def test_chainrule_error(self, configuration, read):
+        with pytest.raises(ChainRuleError):
+            Tags.tagscleanupdicts(failchained=True)
+
+    def test_clean_dataset_tags(self, configuration, read):
+        Tags.set_tagsdict(None)
+        Tags.set_wildcard_tags(None)
+        Tags.tagscleanupdicts(failchained=False)
+        dataset = Dataset.read_from_hdx('TEST1')
+        assert dataset.get_tags() == ['conflict', 'political violence']
+        assert dataset.clean_dataset_tags() == (True, False)
+        dataset.remove_tag('conflicts')
+        assert dataset.get_tags() == ['political violence']
+        assert dataset.clean_dataset_tags() == (False, False)
+        dataset.add_tags(['nodeid123', 'transportation'])
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation']
+        dataset.add_tags(['geodata', 'points'])
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata']
+        dataset.add_tag('financial')
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata', 'finance']
+        dataset.add_tag('addresses')
+        assert dataset.clean_dataset_tags() == (False, True)
+        dataset.remove_tag('addresses')
+        dataset.add_tag('cultivos coca')
+        assert dataset.clean_dataset_tags() == (False, True)
+        dataset.remove_tag('cultivos coca')
+        dataset.add_tag('atentados')
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata', 'finance', 'atentados', 'attacks']
+        dataset.remove_tag('atentados')
+        dataset.remove_tag('attacks')
+        dataset.add_tag('windspeeds')
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata', 'finance', 'wind speed']
+        dataset.add_tag('chil')
+        assert dataset.clean_dataset_tags() == (True, False)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata', 'finance', 'wind speed', 'children']
+        dataset.remove_tag('children')
+        dataset.add_tag('conservancies')
+        assert dataset.clean_dataset_tags() == (False, True)
+        assert dataset.get_tags() == ['political violence', 'transportation', 'geodata', 'finance', 'wind speed', 'conservancies']
+        dataset.remove_tag('conservancies')
+
+    def test_set_quickchart_resource(self, configuration, read):
+        dataset = Dataset.read_from_hdx('TEST1')
+        assert 'dataset_preview' not in dataset
+        assert dataset.set_quickchart_resource('3d777226-96aa-4239-860a-703389d16d1f') is True
+        assert dataset['dataset_preview'] == 'resource_id'
+        resources = dataset.get_resources()
+        assert resources[0]['dataset_preview_enabled'] == 'False'
+        assert resources[1]['dataset_preview_enabled'] == 'True'
+        assert dataset.set_quickchart_resource(resources[0]) is True
+        assert resources[0]['dataset_preview_enabled'] == 'True'
+        assert resources[1]['dataset_preview_enabled'] == 'False'
+        assert dataset.set_quickchart_resource(resources[1].data) is True
+        assert resources[0]['dataset_preview_enabled'] == 'False'
+        assert resources[1]['dataset_preview_enabled'] == 'True'
+        assert dataset.set_quickchart_resource(0) is True
+        assert resources[0]['dataset_preview_enabled'] == 'True'
+        assert resources[1]['dataset_preview_enabled'] == 'False'
+        assert dataset.set_quickchart_resource('12345') is False
+        with pytest.raises(HDXError):
+            dataset.set_quickchart_resource(True)
+        dataset.preview_off()
+        assert dataset['dataset_preview'] == 'no_preview'
+        assert resources[0]['dataset_preview_enabled'] == 'False'
+        assert resources[1]['dataset_preview_enabled'] == 'False'
+        assert dataset.set_quickchart_resource('Resource2') is True
+        assert dataset['dataset_preview'] == 'resource_id'
+        assert resources[0]['dataset_preview_enabled'] == 'False'
+        assert resources[1]['dataset_preview_enabled'] == 'True'
+        assert dataset.set_quickchart_resource({'name': 'Resource1'}) is True
+        assert dataset['dataset_preview'] == 'resource_id'
+        assert resources[0]['dataset_preview_enabled'] == 'True'
+        assert resources[1]['dataset_preview_enabled'] == 'False'
+
+
