@@ -11,75 +11,118 @@ from hdx.hdx_configuration import Configuration, ConfigurationError
 
 class TestConfiguration:
     @pytest.fixture(scope='class')
-    def empty_hdx_key_file(self):
-        return join('tests', 'fixtures', '.emptyhdxkey')
+    def hdx_base_config_yaml(self, configfolder):
+        return join(configfolder, 'hdx_base_config.yml')
 
     @pytest.fixture(scope='class')
-    def hdx_config_yaml(self, configfolder):
-        return join(configfolder, 'hdx_config.yml')
+    def hdx_base_config_json(self, configfolder):
+        return join(configfolder, 'hdx_base_config.json')
 
     @pytest.fixture(scope='class')
     def hdx_config_json(self, configfolder):
         return join(configfolder, 'hdx_config.json')
 
+    @pytest.fixture(scope='class')
+    def hdx_missing_site_config_json(self, configfolder):
+        return join(configfolder, 'hdx_missing_site_config.json')
+
     @pytest.fixture(scope='class')
     def project_config_json(self, configfolder):
         return join(configfolder, 'project_configuration.json')
 
-    def test_init(self, hdx_key_file, hdx_config_yaml, hdx_config_json, project_config_json, project_config_yaml):
-        default_key_file = Configuration.default_hdx_key_file
-        Configuration.default_hdx_key_file = 'NOT EXIST'
-        with pytest.raises(IOError):
+    @pytest.fixture(scope='class')
+    def user_agent_config_yaml(self, configfolder):
+        return join(configfolder, 'user_agent_config.yml')
+
+    @pytest.fixture(scope='class')
+    def user_agent_config2_yaml(self, configfolder):
+        return join(configfolder, 'user_agent_config2.yml')
+
+    @pytest.fixture(scope='class')
+    def user_agent_config3_yaml(self, configfolder):
+        return join(configfolder, 'user_agent_config3.yml')
+
+    @pytest.fixture(scope='class')
+    def empty_yaml(self, configfolder):
+        return join(configfolder, 'empty.yml')
+
+    @pytest.fixture(scope='class')
+    def user_agent_config_wrong_yaml(self, configfolder):
+        return join(configfolder, 'user_agent_config_wrong.yml')
+
+    def test_init(self, hdx_config_json, hdx_config_yaml, hdx_base_config_yaml, hdx_base_config_json, hdx_missing_site_config_json, project_config_json, project_config_yaml):
+        default_config_file = Configuration.default_hdx_config_yaml
+        Configuration.default_hdx_config_yaml = 'NOT EXIST'
+        with pytest.raises(ConfigurationError):
             Configuration()
-        Configuration.default_hdx_key_file = default_key_file
+        Configuration.default_hdx_config_yaml = default_config_file
+
+        Configuration.default_hdx_config_yaml = hdx_config_yaml
+        assert Configuration().get_api_key() == '12345'
+        Configuration.default_hdx_config_yaml = default_config_file
 
         with pytest.raises(IOError):
-            Configuration(hdx_key_file='NOT_EXIST', project_config_yaml=project_config_yaml)
+            Configuration(hdx_config_yaml='NOT_EXIST', project_config_yaml=project_config_yaml)
 
         with pytest.raises(IOError):
-            Configuration(hdx_key_file=hdx_key_file, hdx_config_yaml='NOT_EXIST',
+            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_yaml='NOT_EXIST',
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(IOError):
-            Configuration(hdx_key_file=hdx_key_file, hdx_config_json='NOT_EXIST',
+            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_json='NOT_EXIST',
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, hdx_config_dict={'a': 1}, hdx_config_yaml=hdx_config_yaml,
-                          project_config_yaml=project_config_yaml)
+            Configuration(hdx_config_dict={'a': 1}, hdx_config_yaml=hdx_base_config_yaml,
+                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, hdx_config_dict={'a': 1}, hdx_config_json=hdx_config_json,
-                          project_config_yaml=project_config_yaml)
+            Configuration(hdx_config_dict={'a': 1}, hdx_config_json=hdx_config_json,
+                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, hdx_config_json=hdx_config_json, hdx_config_yaml=hdx_config_yaml,
-                          project_config_yaml=project_config_yaml)
+            Configuration(hdx_config_json=hdx_config_json, hdx_config_yaml=hdx_config_yaml,
+                          hdx_base_config_json = hdx_base_config_json, project_config_yaml = project_config_yaml)
+
+        with pytest.raises(ConfigurationError):
+            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_dict={'a': 1},
+                          hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)
+
+        with pytest.raises(ConfigurationError):
+            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_dict={'a': 1},
+                          hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)
+
+        with pytest.raises(ConfigurationError):
+            Configuration(hdx_config_yaml=hdx_config_yaml, hdx_base_config_json=hdx_base_config_json,
+                          hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)
 
         with pytest.raises(IOError):
-            Configuration(hdx_key_file=hdx_key_file, project_config_yaml='NOT_EXIST')
+            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_yaml='NOT_EXIST')
 
         with pytest.raises(IOError):
-            Configuration(hdx_key_file=hdx_key_file, project_config_json='NOT_EXIST')
+            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_json='NOT_EXIST')
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_site='NOT_EXIST', hdx_key_file=hdx_key_file, project_config_yaml=project_config_yaml)
+            Configuration(hdx_config_json=hdx_missing_site_config_json, project_config_json=project_config_json, hdx_base_config_json=hdx_base_config_json)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, project_config_json=project_config_json,
+            Configuration(hdx_site='NOT_EXIST', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
+
+        with pytest.raises(ConfigurationError):
+            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json,
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, project_config_dict={'la': 'la'},
+            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_key_file=hdx_key_file, project_config_dict={'la': 'la'},
+            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
                           project_config_json=project_config_json)
 
-    def test_hdx_configuration_dict(self, hdx_key_file, project_config_yaml, mocksmtp):
-        Configuration._create(hdx_site='prod', hdx_key_file=hdx_key_file,
-                              hdx_config_dict={
+    def test_hdx_configuration_dict(self, project_config_yaml, mocksmtp):
+        Configuration._create(user_agent='test', hdx_config_dict={'hdx_site': 'prod', 'hdx_read_only': True, 'hdx_key': 'abcde'},
+                              hdx_base_config_dict={
                                                  'hdx_prod_site': {
                                                      'url': 'https://data.humdata.org/',
                                                      'username': None,
@@ -89,8 +132,10 @@ class TestConfiguration:
                                              },
                               project_config_yaml=project_config_yaml)
         expected_configuration = {
-            'api_key': '12345',
-            'param_1': 'ABC',
+            'hdx_site': 'prod',
+            'hdx_read_only': True,
+            'hdx_key': 'abcde',
+            'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -147,12 +192,14 @@ To: larry@gmail.com, moe@gmail.com, curly@gmail.com
 hello there'''
         assert email.server.send_args == {'mail_options': ['a', 'b'], 'rcpt_options': [1, 2]}
 
-    def test_hdx_configuration_json(self, hdx_key_file, hdx_config_json, project_config_yaml):
-        Configuration._create(hdx_key_file=hdx_key_file, hdx_config_json=hdx_config_json,
-                              project_config_yaml=project_config_yaml)
+    def test_hdx_configuration_json(self, hdx_config_json, hdx_base_config_json, project_config_yaml):
+        Configuration._create(user_agent='test', hdx_config_json=hdx_config_json,
+                              hdx_base_config_json=hdx_base_config_json, project_config_yaml=project_config_yaml)
         expected_configuration = {
-            'api_key': '12345',
-            'param_1': 'ABC',
+            'hdx_site': 'test',
+            'hdx_read_only': False,
+            'hdx_key': '54321',
+            'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -172,12 +219,14 @@ hello there'''
         }
         assert Configuration.read() == expected_configuration
 
-    def test_hdx_configuration_yaml(self, hdx_key_file, hdx_config_yaml, project_config_yaml):
-        Configuration._create(hdx_key_file=hdx_key_file,
-                             hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
+    def test_hdx_configuration_yaml(self, hdx_config_yaml, hdx_base_config_yaml, project_config_yaml):
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml,
+                              hdx_base_config_yaml=hdx_base_config_yaml, project_config_yaml=project_config_yaml)
         expected_configuration = {
-            'api_key': '12345',
-            'param_1': 'ABC',
+            'hdx_site': 'prod',
+            'hdx_read_only': False,
+            'hdx_key': '12345',
+            'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -198,10 +247,12 @@ hello there'''
         }
         assert Configuration.read() == expected_configuration
 
-    def test_project_configuration_dict(self, hdx_key_file):
-        Configuration._create(hdx_key_file=hdx_key_file)
+    def test_project_configuration_dict(self, hdx_config_yaml):
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml)
         expected_configuration = {
-            'api_key': '12345',
+            'hdx_site': 'prod',
+            'hdx_read_only': False,
+            'hdx_key': '12345',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -252,8 +303,13 @@ hello there'''
                 'file_types',
                 'num_of_rows'
             ]},
-            'resource': {'required_fields': ['package_id', 'name', 'format', 'url', 'description',
-                                             'url_type', 'resource_type']},
+            'resource': {'required_fields': [
+                'package_id',
+                'name',
+                'format',
+                'description',
+                'url_type',
+                'resource_type']},
             'showcase': {'required_fields': [
                 'name',
                 'title',
@@ -279,16 +335,19 @@ hello there'''
                 'title',
                 'view_type',
             ]},
+            'tags_cleanup_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vTyUP_oS878LihWM7iR0qry0OZP44BKyVXc1P_SoM0FEvbVGdOQxxlQvBWT_vwwGNaAbmuSeTg1FwuP/pub?gid=346948259&single=true&output=csv'
         }
         assert Configuration.read() == expected_configuration
-        Configuration._create(hdx_key_file=hdx_key_file, project_config_dict={'abc': '123'})
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'abc': '123'})
         expected_configuration['abc'] = '123'
         assert Configuration.read() == expected_configuration
 
-    def test_project_configuration_json(self, hdx_key_file, project_config_json):
-        Configuration._create(hdx_key_file=hdx_key_file, project_config_json=project_config_json)
+    def test_project_configuration_json(self, hdx_config_yaml, project_config_json):
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json)
         expected_configuration = {
-            'api_key': '12345',
+            'hdx_site': 'prod',
+            'hdx_read_only': False,
+            'hdx_key': '12345',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -340,9 +399,13 @@ hello there'''
                 'file_types',
                 'num_of_rows'
             ]},
-            'resource': {'required_fields': ['package_id', 'name', 'format', 'url', 'description',
-                                             'url_type', 'resource_type'
-                                             ]},
+            'resource': {'required_fields': [
+                'package_id',
+                'name',
+                'format',
+                'description',
+                'url_type',
+                'resource_type']},
             'showcase': {'required_fields': [
                 'name',
                 'title',
@@ -368,14 +431,17 @@ hello there'''
                 'title',
                 'view_type',
             ]},
+            'tags_cleanup_url': 'https://docs.google.com/spreadsheets/d/e/2PACX-1vTyUP_oS878LihWM7iR0qry0OZP44BKyVXc1P_SoM0FEvbVGdOQxxlQvBWT_vwwGNaAbmuSeTg1FwuP/pub?gid=346948259&single=true&output=csv'
         }
         assert Configuration.read() == expected_configuration
 
-    def test_project_configuration_yaml(self, hdx_key_file, project_config_yaml):
-        Configuration._create(hdx_key_file=hdx_key_file, project_config_yaml=project_config_yaml)
+    def test_project_configuration_yaml(self, hdx_config_yaml, project_config_yaml):
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
         expected_configuration = {
-            'api_key': '12345',
-            'param_1': 'ABC',
+            'hdx_site': 'prod',
+            'hdx_read_only': False,
+            'hdx_key': '12345',
+            'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
                 'username': None,
@@ -426,9 +492,13 @@ hello there'''
                 'file_types',
                 'num_of_rows'
             ]},
-            'resource': {'required_fields': ['package_id', 'name', 'format', 'url', 'description',
-                                             'url_type', 'resource_type'
-                                             ]},
+            'resource': {'required_fields': [
+                'package_id',
+                'name',
+                'format',
+                'description',
+                'url_type',
+                'resource_type']},
             'showcase': {'required_fields': [
                 'name',
                 'title',
@@ -457,28 +527,23 @@ hello there'''
         }
         assert Configuration.read() == expected_configuration
 
-    def test_get_hdx_key_site(self, hdx_key_file, project_config_yaml):
-        Configuration._create(hdx_site='prod', hdx_key_file=hdx_key_file,
-                              hdx_config_dict={},
-                              project_config_yaml=project_config_yaml)
+    def test_get_hdx_key_site(self, hdx_config_yaml, project_config_yaml):
+        Configuration._create(user_agent='test', hdx_config_yaml=hdx_config_yaml,
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         actual_configuration = Configuration.read()
         assert actual_configuration.get_api_key() == '12345'
         assert actual_configuration.get_hdx_site_url() == 'https://data.humdata.org/'
         assert actual_configuration._get_credentials() == ('', '')
 
-    def test_set_hdx_key_value(self, empty_hdx_key_file, project_config_yaml):
-        with pytest.raises(LoadError):
-            Configuration.load_api_key(empty_hdx_key_file)
-        Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                              hdx_config_dict={},
-                              project_config_yaml=project_config_yaml)
+    def test_set_hdx_key_value(self, project_config_yaml):
+        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         configuration = Configuration.read()
         assert configuration.get_api_key() == 'TEST_HDX_KEY'
         configuration.set_api_key('NEW API KEY')
         assert configuration.get_api_key() == 'NEW API KEY'
-        Configuration._create(hdx_site='prod', hdx_read_only=True,
-                              hdx_config_dict={},
-                              project_config_yaml=project_config_yaml)
+        Configuration._create(user_agent='test', hdx_site='prod', hdx_read_only=True,
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         assert Configuration.read().get_api_key() is None
         configuration = Configuration.read()
         configuration.set_api_key('TEST API KEY')
@@ -492,39 +557,34 @@ hello there'''
         assert configuration.get_api_key() == 'NEW API KEY'
 
     def test_create_set_configuration(self, project_config_yaml):
-        Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                              hdx_config_dict={},
-                              project_config_yaml=project_config_yaml)
+        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         with pytest.raises(ConfigurationError):
-            Configuration.create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                                 hdx_config_dict={},
-                                 project_config_yaml=project_config_yaml)
-        configuration = Configuration(hdx_site='test', hdx_key='OTHER_TEST_HDX_KEY',
-                                      hdx_config_dict={},
-                                      project_config_yaml=project_config_yaml)
+            Configuration.create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                                 hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        configuration = Configuration(user_agent='test', hdx_site='test', hdx_key='OTHER_TEST_HDX_KEY',
+                                      hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         Configuration.setup(configuration)
         assert Configuration.read() == configuration
         Configuration.delete()
         with pytest.raises(ConfigurationError):
             Configuration.read()
-        Configuration.create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                             hdx_config_dict={},
-                             project_config_yaml=project_config_yaml)
+        Configuration.create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                             hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         assert Configuration.read().get_api_key() == 'TEST_HDX_KEY'
 
     def test_remoteckan_validlocations(self, project_config_yaml):
-        Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                              hdx_config_dict={},
-                              project_config_yaml=project_config_yaml)
+        Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         remoteckan = ckanapi.RemoteCKAN('http://lalala', apikey='12345',
                                         user_agent='HDXPythonLibrary/1.0')
-        Configuration.read().setup_remoteckan(remoteckan)
+        Configuration.read().setup_remoteckan(remoteckan=remoteckan)
         assert Configuration.read().remoteckan() == remoteckan
         remoteckan = ckanapi.RemoteCKAN('http://hahaha', apikey='54321',
                                         user_agent='HDXPythonLibrary/0.5')
         Configuration._create(remoteckan=remoteckan,
                               hdx_site='prod', hdx_key='TEST_HDX_KEY',
-                              hdx_config_dict={},
+                              hdx_base_config_dict={},
                               project_config_yaml=project_config_yaml)
         assert Configuration.read().remoteckan() == remoteckan
         Configuration.read()._remoteckan = None
@@ -533,3 +593,35 @@ hello there'''
         Configuration.delete()
         with pytest.raises(ConfigurationError):
             Configuration.read().remoteckan()
+
+    def test_user_agent(self, user_agent_config_yaml, user_agent_config2_yaml, user_agent_config3_yaml,
+                        empty_yaml, user_agent_config_wrong_yaml, project_config_yaml):
+        Configuration._create(user_agent_config_yaml=user_agent_config_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        version = Configuration.get_version()
+        assert Configuration.read().remoteckan().user_agent == 'lala:HDXPythonLibrary/%s-myua' % version
+        Configuration._create(user_agent_config_yaml=user_agent_config2_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-myuseragent' % version
+        Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='lookup',  hdx_site='prod',
+                              hdx_key='TEST_HDX_KEY', hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-mylookupagent' % version
+        Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='lookup2',  hdx_site='prod',
+                              hdx_key='TEST_HDX_KEY', hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        assert Configuration.read().remoteckan().user_agent == 'HDXPythonLibrary/%s-mylookupagent2' % version
+        Configuration._create(user_agent='my_ua', preprefix='papa', hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                              hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        assert Configuration.read().remoteckan().user_agent == 'papa:HDXPythonLibrary/%s-my_ua' % version
+        with pytest.raises(ConfigurationError):
+            Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='fail',
+                                  hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        with pytest.raises(LoadError):
+            Configuration._create(user_agent_config_yaml=empty_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        with pytest.raises(ConfigurationError):
+            Configuration._create(user_agent_config_yaml=user_agent_config_wrong_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+        with pytest.raises(ConfigurationError):
+            Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
+                                  hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
